# -*- encoding: utf-8 -*-

from django.utils.translation import ugettext_lazy as _
from django.contrib.sites.models import Site
from django.db import models


class AbstractCalendarSMS(models.Model):
    status = models.BooleanField(_('Status'), default=True)
    created = models.DateTimeField(
        _('Created'), auto_now=False, auto_now_add=True,
        db_index=True, editable=False)
    updated = models.DateTimeField(
        _('Updated'), auto_now=True, auto_now_add=True,
        db_index=True, editable=False)

    class Meta:
        abstract = True


class CalendarSMSWebsites(AbstractCalendarSMS):
    site = models.ForeignKey(Site, verbose_name=_('Website'), default=1)

    def __unicode__(self):
        return self.site.domain

    class Meta:
        db_table = 'calendar_sms_websites'
        verbose_name = _('Websites')
        verbose_name_plural = _('Websites')


class CalendarSMSSettings(AbstractCalendarSMS):
    email = models.CharField(_('Google account'), max_length=250)
    password = models.CharField(
        _('Account password'), max_length=250,
        help_text=_('That is application password, not from a mail account'))
    calendar = models.CharField(
        _('Calendar ID'), max_length=250, blank=True, null=True,
        help_text=_('For example: fafgttkq8ed2idfgd@group.calendar.google.com.'
                    'You can view the calendar settings - details specific '
                    'calendar field "Calendar Address" and right there in '
                    'the brackets.'))
    start_time = models.PositiveIntegerField(
        _('Event starts'), default=120, max_length=3,
        help_text=_('Specifies how long the event will begin (in seconds)'))
    end_time = models.PositiveIntegerField(
        _('End of event'), default=180, max_length=3,
        help_text=_(
            'Specifies how long the event will be complete (in seconds)'
        ))
    reminder_time = models.PositiveIntegerField(
        _('Reminder time'), default=1, max_length=2,
        help_text=_('Reminder time in minutes'))
    attempts = models.PositiveIntegerField(
        _('Numbers of attempts'), default=2, max_length=2,
        help_text=_('Max number of attempts to establish a new '
                    'memo to the calendar'))
    delays = models.PositiveIntegerField(
        _('Delays'), default=5, max_length=1,
        help_text=_('Waiting time between attempts'))
    website = models.ForeignKey(CalendarSMSWebsites)

    def __unicode__(self):
        return self.email

    class Meta:
        db_table = 'calendar_sms_settings'


class CalendarSMSLogs(AbstractCalendarSMS):
    title = models.CharField(_('Title'), max_length=160)
    content = models.CharField(
        _('Message'), max_length=350, blank=True, null=True)
    email = models.ForeignKey(CalendarSMSSettings, verbose_name=_('Email'))

    def __unicode__(self):
        return '%s / %s' % (self.email, self.title)

    class Meta:
        db_table = 'calendar_sms_logs'
        ordering = ['-created']
        verbose_name = _('SMS Logs')
        verbose_name_plural = _('SMS Logs')



# -*- coding: utf-8 -*-

from time import strftime, gmtime, time, sleep
from gdata.calendar.service import *

from django.forms.models import model_to_dict
from django.contrib.sites.models import Site

from models import CalendarSMSSettings, CalendarSMSLogs


class DjangoCalendarSMS(object):

    def __init__(self):

        self.account = None
        self.email = None
        self.password = None
        self.calendar = None
        self.start_time = None
        self.end_time = None
        self.reminder_time = None
        self.attempts = None
        self.delays = None

        self.default_cal = '/calendar/feeds/default/private/full'
        self.user_cal = 'https://www.google.com/calendar/feeds/%s/private/full'

        self.agent = 'Google-Calendar_SMS-2.2'
        self.debug = False

    def __message(self, msg):
        if self.debug:
            print(msg)

    def _insertEvent(self):
        event = gdata.calendar.CalendarEventEntry()
        event.title = atom.Title(text=self.title)
        event.content = atom.Content(text=self.content)
        event.where.append(gdata.calendar.Where(value_string=self.where))
        start_time = strftime(
            '%Y-%m-%dT%H:%M:%S.000Z', gmtime(time() + self.start_time)
        )
        end_time = strftime(
            '%Y-%m-%dT%H:%M:%S.000Z', gmtime(time() + self.end_time)
        )
        event.when.append(gdata.calendar.When(
            start_time=start_time, end_time=end_time
        ))
        if not self.calendar:
            new_event = self.cal_client.InsertEvent(event, self.default_cal)
        else:
            new_event = self.cal_client.InsertEvent(
                event, self.user_cal % self.calendar)
        self.edit_link = new_event.GetEditLink().href
        return new_event

    def _addReminder(self, event):
        for a_when in event.when:
            if len(a_when.reminder) > 0:
                a_when.reminder[0].minutes = self.reminder_time
            else:
                a_when.reminder.append(
                    gdata.calendar.Reminder(minutes=self.reminder_time)
                )
        for a_when in event.when:
            if len(a_when.reminder) > 0:
                a_when.reminder[0].method = 'sms'
            else:
                a_when.reminder.append(gdata.calendar.Reminder(method='sms'))
        return self.cal_client.UpdateEvent(event.GetEditLink().href, event)

    def __remove(self):
        try:
            self.cal_client.DeleteEvent(self.edit_link)
        except Exception, msg:
            self.__message(msg)

    def __add(self):
        self.cal_client = CalendarService(
            self.email, self.password, self.agent
        )
        self.cal_client.ProgrammaticLogin()
        self._addReminder(self._insertEvent())

    def __save(self, status):
        CalendarSMSLogs.objects.create(
            title=self.title, email=self.account,
            content=self.content, status=status)

    def __send(self):
        for i in range(self.attempts):
            try:
                self.__message('Sending sms to %s' % self.email)
                self.__add()
                self.__save(True)
                self.__message('successfully')
                break
            except Exception, msg:
                if self.attempts == (i + 1):
                    self.__save(False)
                self.__remove()
                self.__message(msg)
            sleep(self.delays)
            del self.cal_client
            self.__message('attempt %d' % (i + 1))

    def __run(self):
        for self.account in self.accounts:
            self.__dict__.update(model_to_dict(self.account))
            if self.email and self.password:
                self.__send()

    def __get_accounts(self):
        self.accounts = CalendarSMSSettings.objects.filter(
            website__site=Site.objects.get_current(),
            website__status=True,
            status=True
        )
        return self.accounts.exists()

    def send(self, title='', content='', where=''):
        self.title = title
        self.content = content
        self.where = where

        if self.__get_accounts() and title:
            self.__run()
        else:
            self.__message('Active accounts not found')


def sendSMS(*args, **kwargs):
    sms = DjangoCalendarSMS()
    sms.send(*args, **kwargs)



# -*- coding: utf-8 -*-

from django import forms

from models import CalendarSMSSettings


class CalendarSMSSettingsForm(forms.ModelForm):
    class Meta:
        model = CalendarSMSSettings
        widgets = {
            'password': forms.PasswordInput()
        }


# -*- coding: utf-8 -*-

from django.contrib import admin

from models import (CalendarSMSLogs, CalendarSMSWebsites, CalendarSMSSettings)
from forms import CalendarSMSSettingsForm


class CalendarSMSSettingsInline(admin.TabularInline):
    model = CalendarSMSSettings
    extra = 1
    form = CalendarSMSSettingsForm


class CalendarSMSLogsAdmin(admin.ModelAdmin):
    search_fields = ('title', 'message',)
    list_display = (
        'email', 'title', 'content', 'status', 'created', 'updated',
    )
    list_display_links = ('title',)
    list_filter = ('status', 'created', 'updated', 'email__website', 'email')
    ordering = ('created',)

    def has_add_permission(self, request, obj=None):
        return False

    def __init__(self, model, admin_site):
        admin.ModelAdmin.__init__(self, model, admin_site)
        self.readonly_fields = [field.name for field in model._meta.fields]
        self.readonly_model = model


class CalendarSMSWebsitesAdmin(admin.ModelAdmin):
    search_fields = ('website',)
    list_display = (
        'site', 'status', 'created', 'updated', 'id',
    )
    list_display_links = ('site',)
    list_filter = ('status', 'created', 'updated')
    fields = ('site', 'status',)
    ordering = ('created',)
    inlines = (CalendarSMSSettingsInline,)


admin.site.register(CalendarSMSLogs, CalendarSMSLogsAdmin)
admin.site.register(CalendarSMSWebsites, CalendarSMSWebsitesAdmin)



from setuptools import setup
from calendar_sms import VERSION


setup(
    name='django-calendar-sms',
    version=".".join(map(str, VERSION)),
    description='This reusable Django app can help you to send sms via '
                'Google Calendar.',
    keywords="django sms",
    long_description=open('README').read(),
    author="GoTLiuM InSPiRiT",
    author_email='gotlium@gmail.com',
    url='http://github.com/gotlium/django-calendar-sms',
    packages=['calendar_sms'],
    include_package_data=True,
    install_requires=['setuptools', 'gdata', 'django'],
    zip_safe=False,
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
)
